/* Copyright 2020-2024, Michele Santullo
 * This file is part of wrenpp.
 *
 * Wrenpp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wrenpp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with wrenpp.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "wrenpp/handle.hpp"
#include <utility>

namespace wren {

template <typename T>
class ForeignObject {
public:
	ForeignObject() = default;
	ForeignObject (std::nullptr_t) : ForeignObject() {}
	ForeignObject (T* obj, Handle&& handle);
	ForeignObject (ForeignObject&&) = default;
	~ForeignObject() noexcept = default;

	ForeignObject& operator= (ForeignObject&&) = default;
	bool operator!= (std::nullptr_t) const { return nullptr != m_object; }
	bool operator== (std::nullptr_t) const { return nullptr == m_object; }

	void set_to_slot (int num);
	T& object() { return *m_object; }
	const T& object() const { return *m_object; }
	Handle& handle() { return m_handle; }
	const Handle& handle() const { return m_handle; }
	bool is_valid() const { return nullptr != m_object; }

	operator T&() { return object(); }
	operator const T&() const { return object(); }
	operator Handle&() { return handle(); }
	operator const Handle&() const { return handle(); }

private:
	Handle m_handle;
	T* m_object{nullptr};
};

template <typename T>
inline ForeignObject<T>::ForeignObject (T* obj, Handle&& handle) :
	m_handle(std::move(handle)),
	m_object(obj)
{ }

template <typename T>
inline void ForeignObject<T>::set_to_slot (int num) {
	m_handle.set_to_slot(num);
}

} //namespace wren
